"use strict";
var utils_1 = require("../utils");
var path = require("path");
var semver = require("semver");
var fs = require("fs");
var xmldom_1 = require("xmldom");
var conventionalChangelog = require("conventional-changelog");
var simpleGit = require("simple-git");
function getNextVersion(options, version) {
    var nextVersion;
    if (options["use-version"]) {
        if (semver.valid(options["use-version"])) {
            utils_1.logger.error(options["use-version"] + " is no valid version! Aborting release...");
            process.exit(1);
        }
        else {
            nextVersion = options["use-version"];
        }
    }
    else {
        nextVersion = semver.inc(version, options.major ? "major" : options.minor ? "minor" : "patch");
    }
    return nextVersion;
}
function updateConfigXml(configPath, version) {
    var configXmlFile = fs.readFileSync(configPath, "utf-8");
    var configXmlDom = new xmldom_1.DOMParser().parseFromString(configXmlFile, "application/xml");
    var widgetElement = configXmlDom.getElementsByTagName("widget")[0];
    widgetElement.setAttribute("version", version);
    if (widgetElement.hasAttribute("android-versionCode")) {
        widgetElement.setAttribute("android-versionCode", String(Number(widgetElement.getAttribute("android-versionCode")) + 1));
    }
    var serializedConfigXml = new xmldom_1.XMLSerializer().serializeToString(configXmlDom);
    fs.writeFileSync(configPath, serializedConfigXml, "utf-8");
    utils_1.logger.info("Updated config.xml file.");
}
function updateChangelog(rootDir, changelogFileName) {
    var changelogPath = path.join(rootDir, changelogFileName);
    if (!fs.existsSync(changelogPath)) {
        fs.writeFileSync(changelogPath, "", {
            encoding: "utf-8"
        });
    }
    return new Promise(function (resolve) {
        var changelogEntry = "";
        conventionalChangelog({
            preset: "angular"
        }).on("data", function (data) {
            changelogEntry += data.toString();
        }).on("end", function () {
            var existingChangelog = fs.readFileSync(changelogPath);
            var changelog = changelogEntry + existingChangelog;
            fs.writeFileSync(changelogPath, changelog, { encoding: "utf-8" });
            utils_1.logger.info("Updated " + changelogFileName + " file.");
            resolve(0);
        });
    });
}
exports.releaseCommand = {
    command: "release",
    describe: "Perform a release. Update the version in the package.json and in an cordova project aswell in the config.xml.",
    builder: {
        "patch": {
            description: "Perform a patch release.",
            type: "boolean"
        },
        "minor": {
            description: "Perform a minor release.",
            type: "boolean"
        },
        "major": {
            description: "Perform a major release.",
            type: "boolean"
        },
        "use-version": {
            description: "Release the given version.",
            type: "string"
        }
    },
    handler: function (options) {
        var rootDir = process.cwd();
        var pkgPath = path.join(rootDir, "package.json");
        var pkg = require(pkgPath);
        var version = pkg.version;
        var nextVersion = getNextVersion(options, version);
        pkg.version = nextVersion;
        fs.writeFileSync(pkgPath, JSON.stringify(pkg, null, 2), "utf-8");
        utils_1.logger.info("Updated package.json.");
        var changelogFileName = "CHANGELOG.md";
        updateChangelog(rootDir, changelogFileName).then(function () {
            var filesToAdd = ["package.json", changelogFileName];
            var configPath = path.join(rootDir, "config.xml");
            var hasConfigXmlFile = fs.existsSync(configPath);
            if (hasConfigXmlFile) {
                updateConfigXml(configPath, nextVersion);
                filesToAdd.push("config.xml");
            }
            return simpleGit(rootDir)
                .add(filesToAdd)
                .commit(nextVersion)
                .addTag(nextVersion, function (error) {
                if (error) {
                    process.exit(1);
                }
                else {
                    process.exit(0);
                }
            });
        });
    }
};
//# sourceMappingURL=release.command.js.map