"use strict";
var path = require("path");
var webpack = require("webpack");
var webpackMerge = require("webpack-merge");
var webpack_config_common_1 = require("../webpack/webpack.config.common");
var webpack_config_dll_1 = require("../webpack/webpack.config.dll");
var fs = require("fs");
var webpack_vendors_1 = require("../webpack/webpack.vendors");
var webpack_polyfills_1 = require("../webpack/webpack.polyfills");
var utils_1 = require("../utils");
var DLL_CACHE_FILE_NAME = "dll.cache.json";
module.exports = function (env, config, forceUpdate) {
    var pkg = JSON.parse(fs.readFileSync(path.join(config.rootDir, "package.json"), "utf-8"));
    var dllDependencies = getDllDependencies(config);
    if (forceUpdate) {
        utils_1.logger.debug("Forced DLL cache update.");
        return updateDlls(env, config, dllDependencies, pkg);
    }
    else {
        if (dllsUpToDate(config.dllDir, DLL_CACHE_FILE_NAME, pkg, dllDependencies)) {
            utils_1.logger.info("Found up-to-date DLL cache.");
            return Promise.resolve(0);
        }
        else {
            return updateDlls(env, config, dllDependencies, pkg);
        }
    }
};
function updateDlls(env, config, dllDependencies, pkg) {
    utils_1.logger.info("Updating DLL cache...");
    var indexPath = path.join(config.sourceDir, "index.html");
    var commonConfig = webpack_config_common_1.getCommonConfigPartial(indexPath, env, config, false, false);
    var buildConfig = webpack_config_dll_1.getDllConfigPartial(path.join(config.dllDir));
    var webpackConfig = webpackMerge.strategy({
        "entry": "replace"
    })(commonConfig, buildConfig);
    return runBuild(webpackConfig).then(function (exitCode) {
        if (exitCode === 0) {
            writeDllCache(config.dllDir, DLL_CACHE_FILE_NAME, dllDependencies, pkg);
        }
    });
}
function runBuild(config) {
    utils_1.logger.info("Creating DLL files and manifests...");
    utils_1.timer.start("DLL bundle creation");
    return new Promise(function (resolve, reject) {
        webpack(config, function (error, stats) {
            if (stats.hasErrors()) {
                return reject(stats.toJson().errors[0]);
            }
            utils_1.timer.end("DLL bundle creation");
            resolve(0);
        });
    });
}
function getDllDependencies(config) {
    var additionalVendors = config.typescript && Array.isArray(config.typescript.vendors) ? config.typescript.vendors : [];
    return webpack_vendors_1.VENDORS.concat(webpack_polyfills_1.POLYFILLS).concat(additionalVendors).map(function (vendor) {
        // This is needed to compare vendors with deep import paths like
        // zone.js/dist/long-stack-trace-zone.js. In this case we have to check whether
        // the version of zone.js has changed.
        if (vendor.includes("/")) {
            // Scoped npm packages contain a '/' in their name that we have to preserve
            if (vendor.startsWith("@")) {
                return vendor.split("/")[0] + "/" + vendor.split("/")[1];
            }
            else {
                return vendor.split("/")[0];
            }
        }
        else {
            return vendor;
        }
    });
}
exports.getDllDependencies = getDllDependencies;
function dllsUpToDate(dllDir, dllCacheFile, pkg, dllDependencies) {
    var dllCacheFilePath = path.join(dllDir, dllCacheFile);
    var dllCacheExists = fs.existsSync(dllCacheFilePath);
    if (!dllCacheExists) {
        return false;
    }
    else {
        var dllCacheFileContent = fs.readFileSync(dllCacheFilePath, "utf-8");
        try {
            var dllCache = JSON.parse(dllCacheFileContent);
            var result = true;
            for (var _i = 0, dllDependencies_1 = dllDependencies; _i < dllDependencies_1.length; _i++) {
                var dep = dllDependencies_1[_i];
                result = result && (pkg.dependencies[dep] === dllCache.dependencies[dep]);
            }
            return result;
        }
        catch (error) {
            fs.unlinkSync(dllCacheFilePath);
            return false;
        }
    }
}
exports.dllsUpToDate = dllsUpToDate;
function writeDllCache(dllDir, dllCacheFile, dllDependencies, pkg) {
    var dllCachePath = path.join(dllDir, dllCacheFile);
    var dllCache = {
        dependencies: {}
    };
    for (var _i = 0, dllDependencies_2 = dllDependencies; _i < dllDependencies_2.length; _i++) {
        var dep = dllDependencies_2[_i];
        dllCache.dependencies[dep] = pkg.dependencies[dep];
    }
    if (!fs.existsSync(dllDir)) {
        fs.mkdirSync(dllDir);
    }
    fs.writeFileSync(dllCachePath, JSON.stringify(dllCache, null, 2), { encoding: "utf-8" });
}
exports.writeDllCache = writeDllCache;
//# sourceMappingURL=dll.task.js.map